@@ -87,6 +87,9 @@
 #include "clutter-gst-video-sink.h"
 #include "clutter-gst-private.h"
 
+GST_DEBUG_CATEGORY_STATIC (clutter_gst_video_sink_debug);
+#define GST_CAT_DEFAULT clutter_gst_video_sink_debug
+
 #define CLUTTER_GST_DEFAULT_PRIORITY G_PRIORITY_HIGH_IDLE
 
 #define BASE_SINK_CAPS "{ AYUV,"                \
@@ -98,12 +101,19 @@
   "BGR,"                                        \
   "NV12 }"
 
+
+#define MAKE_CAPS(feature, caps) \
+  GST_VIDEO_CAPS_MAKE_WITH_FEATURES(feature "," GST_CAPS_FEATURE_META_GST_VIDEO_OVERLAY_COMPOSITION, caps) \
+  ";" \
+  GST_VIDEO_CAPS_MAKE_WITH_FEATURES(feature, caps)
+
 static const char clutter_gst_video_sink_caps_str[] =
-  GST_VIDEO_CAPS_MAKE_WITH_FEATURES(GST_CAPS_FEATURE_MEMORY_SYSTEM_MEMORY,
-                                    BASE_SINK_CAPS)
+  MAKE_CAPS (GST_CAPS_FEATURE_MEMORY_SYSTEM_MEMORY, BASE_SINK_CAPS)
+#ifdef HAVE_GL_TEXTURE_UPLOAD
   ";"
-  GST_VIDEO_CAPS_MAKE_WITH_FEATURES(GST_CAPS_FEATURE_META_GST_VIDEO_GL_TEXTURE_UPLOAD_META,
-                                    "RGBA");
+  MAKE_CAPS (GST_CAPS_FEATURE_META_GST_VIDEO_GL_TEXTURE_UPLOAD_META, BASE_SINK_CAPS)
+#endif
+;
 
 
   static GstStaticPadTemplate sinktemplate_all =
@@ -688,7 +698,7 @@ clutter_gst_video_sink_attach_frame (ClutterGstVideoSink *sink,
   ClutterGstVideoSinkPrivate *priv = sink->priv;
   guint i;
 
-  for (i = 0; i < G_N_ELEMENTS (priv->frame); i++)
+  for (i = 0; i < priv->renderer->n_layers; i++)
     if (priv->frame[i] != NULL)
       cogl_pipeline_set_layer_texture (pln, i + priv->video_start,
                                        priv->frame[i]);
@@ -742,6 +752,8 @@ clutter_gst_video_sink_setup_balance (ClutterGstVideoSink *sink,
                                         { 256, 256 } };
       SnippetCacheEntry *entry = get_layer_cache_entry (sink, &snippet_cache);
 
+      GST_INFO_OBJECT (sink, "attaching shader color correction");
+
       if (entry == NULL)
         {
           gchar *source = g_strdup_printf (color_balance_shader,
@@ -791,6 +803,8 @@ clutter_gst_video_sink_setup_balance (ClutterGstVideoSink *sink,
     }
   else
     {
+      GST_INFO_OBJECT (sink, "attaching null color correction");
+
       if (G_UNLIKELY (no_color_balance_snippet_vert == NULL))
         {
           no_color_balance_snippet_vert =
@@ -1138,8 +1152,8 @@ static ClutterGstRenderer rgb24_glsl_renderer =
     CLUTTER_GST_RGB24,
     CLUTTER_GST_RENDERER_NEEDS_GLSL,
 
-    GST_STATIC_CAPS (GST_VIDEO_CAPS_MAKE_WITH_FEATURES (GST_CAPS_FEATURE_MEMORY_SYSTEM_MEMORY,
-                                                        "{ RGB, BGR }")),
+    GST_STATIC_CAPS (MAKE_CAPS (GST_CAPS_FEATURE_MEMORY_SYSTEM_MEMORY,
+                                "{ RGB, BGR }")),
     1, /* n_layers */
     clutter_gst_rgb24_glsl_setup_pipeline,
     clutter_gst_rgb24_upload,
@@ -1152,8 +1166,8 @@ static ClutterGstRenderer rgb24_renderer =
     "RGB 24",
     CLUTTER_GST_RGB24,
     0,
-    GST_STATIC_CAPS (GST_VIDEO_CAPS_MAKE_WITH_FEATURES (GST_CAPS_FEATURE_MEMORY_SYSTEM_MEMORY,
-                                                        "{ RGB, BGR }")),
+    GST_STATIC_CAPS (MAKE_CAPS (GST_CAPS_FEATURE_MEMORY_SYSTEM_MEMORY,
+                                "{ RGB, BGR }")),
     1, /* n_layers */
     clutter_gst_rgb24_setup_pipeline,
     clutter_gst_rgb24_upload,
@@ -1249,6 +1263,8 @@ clutter_gst_rgb32_upload (ClutterGstVideoSink *sink,
   }
 }
 
+static CoglUserDataKey gl_upload_texture_private_key;
+#define MAX_ALLOCATED_GL_TEXTURES (1)
 
 static gboolean
 clutter_gst_rgb32_upload_gl (ClutterGstVideoSink *sink,
@@ -1256,6 +1272,7 @@ clutter_gst_rgb32_upload_gl (ClutterGstVideoSink *sink,
 {
   ClutterGstVideoSinkPrivate *priv = sink->priv;
   GstVideoGLTextureUploadMeta *upload_meta;
+  gint i;
   guint gl_handle[1];
 
   //clear_frame_textures (sink);
@@ -1274,16 +1291,33 @@ clutter_gst_rgb32_upload_gl (ClutterGstVideoSink *sink,
 
   if (priv->frame[0] == NULL)
     {
-      priv->frame[0] = COGL_TEXTURE (cogl_texture_2d_new_with_size (priv->ctx,
-                                                                    priv->info.width,
-                                                                    priv->info.height));
-      cogl_texture_set_components (priv->frame[0], COGL_TEXTURE_COMPONENTS_RGBA);
+      for (i = 0; i < MAX_ALLOCATED_GL_TEXTURES; i++) {
+        priv->frame[i] = COGL_TEXTURE (cogl_texture_2d_new_with_size (priv->ctx,
+                                                                      priv->info.width,
+                                                                      priv->info.height));
+        cogl_texture_set_components (priv->frame[i], COGL_TEXTURE_COMPONENTS_RGBA);
 
-      if (!cogl_texture_allocate (priv->frame[0], NULL)) {
-        GST_WARNING ("Couldn't allocate cogl texture");
-        return FALSE;
+        if (!cogl_texture_allocate (priv->frame[i], NULL)) {
+          GST_WARNING ("Couldn't allocate cogl texture");
+          return FALSE;
+        }
       }
     }
+  else
+    {
+      CoglTexture *tmp;
+
+      tmp = priv->frame[0];
+      for (i = 0; i < MAX_ALLOCATED_GL_TEXTURES - 1; i++)
+        priv->frame[i] = priv->frame[i + 1];
+      priv->frame[MAX_ALLOCATED_GL_TEXTURES - 1] = tmp;
+    }
+
+  cogl_object_set_user_data (COGL_OBJECT (priv->frame[0]),
+                             &gl_upload_texture_private_key,
+                             gst_buffer_ref (buffer),
+                             (CoglUserDataDestroyCallback) gst_buffer_unref);
+
 
   if (!cogl_texture_get_gl_texture (priv->frame[0], &gl_handle[0], NULL)) {
     GST_WARNING ("Couldn't get gl texture");
@@ -1303,11 +1337,14 @@ static ClutterGstRenderer rgb32_glsl_renderer =
     "RGB 32",
     CLUTTER_GST_RGB32,
     CLUTTER_GST_RENDERER_NEEDS_GLSL,
-    GST_STATIC_CAPS (GST_VIDEO_CAPS_MAKE_WITH_FEATURES(GST_CAPS_FEATURE_META_GST_VIDEO_GL_TEXTURE_UPLOAD_META,
-                                                           "RGBA")
+    GST_STATIC_CAPS (
+#ifdef HAVE_GL_TEXTURE_UPLOAD
+                     MAKE_CAPS (GST_CAPS_FEATURE_META_GST_VIDEO_GL_TEXTURE_UPLOAD_META,
+                                "RGBA")
                      ";"
-                     GST_VIDEO_CAPS_MAKE_WITH_FEATURES(GST_CAPS_FEATURE_MEMORY_SYSTEM_MEMORY,
-                                                       "{ RGBA, BGRA }")),
+#endif
+                     MAKE_CAPS (GST_CAPS_FEATURE_MEMORY_SYSTEM_MEMORY,
+                                "{ RGBA, BGRA }")),
     1, /* n_layers */
     clutter_gst_rgb32_glsl_setup_pipeline,
     clutter_gst_rgb32_upload,
@@ -1320,8 +1357,8 @@ static ClutterGstRenderer rgb32_renderer =
     "RGB 32",
     CLUTTER_GST_RGB32,
     0,
-    GST_STATIC_CAPS (GST_VIDEO_CAPS_MAKE_WITH_FEATURES(GST_CAPS_FEATURE_MEMORY_SYSTEM_MEMORY,
-                                                       "{ RGBA, BGRA }")),
+    GST_STATIC_CAPS (MAKE_CAPS (GST_CAPS_FEATURE_MEMORY_SYSTEM_MEMORY,
+                                "{ RGBA, BGRA }")),
     2, /* n_layers */
     clutter_gst_rgb32_setup_pipeline,
     clutter_gst_rgb32_upload,
@@ -1469,8 +1506,8 @@ static ClutterGstRenderer yv12_glsl_renderer =
     "YV12 glsl",
     CLUTTER_GST_YV12,
     CLUTTER_GST_RENDERER_NEEDS_GLSL,
-    GST_STATIC_CAPS (GST_VIDEO_CAPS_MAKE_WITH_FEATURES(GST_CAPS_FEATURE_MEMORY_SYSTEM_MEMORY,
-                                                       "YV12")),
+    GST_STATIC_CAPS (MAKE_CAPS (GST_CAPS_FEATURE_MEMORY_SYSTEM_MEMORY,
+                                "YV12")),
     3, /* n_layers */
     clutter_gst_yv12_glsl_setup_pipeline,
     clutter_gst_yv12_upload,
@@ -1483,8 +1520,8 @@ static ClutterGstRenderer i420_glsl_renderer =
     "I420 glsl",
     CLUTTER_GST_I420,
     CLUTTER_GST_RENDERER_NEEDS_GLSL,
-    GST_STATIC_CAPS (GST_VIDEO_CAPS_MAKE_WITH_FEATURES(GST_CAPS_FEATURE_MEMORY_SYSTEM_MEMORY,
-                                                       "I420")),
+    GST_STATIC_CAPS (MAKE_CAPS (GST_CAPS_FEATURE_MEMORY_SYSTEM_MEMORY,
+                                "I420")),
     3, /* n_layers */
     clutter_gst_yv12_glsl_setup_pipeline,
     clutter_gst_i420_upload,
@@ -1567,8 +1604,8 @@ static ClutterGstRenderer ayuv_glsl_renderer =
     "AYUV glsl",
     CLUTTER_GST_AYUV,
     CLUTTER_GST_RENDERER_NEEDS_GLSL,
-    GST_STATIC_CAPS (GST_VIDEO_CAPS_MAKE_WITH_FEATURES(GST_CAPS_FEATURE_MEMORY_SYSTEM_MEMORY,
-                                                       "AYUV")),
+    GST_STATIC_CAPS (MAKE_CAPS(GST_CAPS_FEATURE_MEMORY_SYSTEM_MEMORY,
+                               "AYUV")),
     1, /* n_layers */
     clutter_gst_ayuv_glsl_setup_pipeline,
     clutter_gst_ayuv_upload,
@@ -1662,8 +1699,8 @@ static ClutterGstRenderer nv12_glsl_renderer =
     "NV12 glsl",
     CLUTTER_GST_NV12,
     CLUTTER_GST_RENDERER_NEEDS_GLSL | CLUTTER_GST_RENDERER_NEEDS_TEXTURE_RG,
-    GST_STATIC_CAPS (GST_VIDEO_CAPS_MAKE_WITH_FEATURES(GST_CAPS_FEATURE_MEMORY_SYSTEM_MEMORY,
-                                                       "NV12")),
+    GST_STATIC_CAPS (MAKE_CAPS(GST_CAPS_FEATURE_MEMORY_SYSTEM_MEMORY,
+                               "NV12")),
     2, /* n_layers */
     clutter_gst_nv12_glsl_setup_pipeline,
     clutter_gst_nv12_upload,
@@ -1760,10 +1797,14 @@ clutter_gst_video_sink_get_caps (GstBaseSink *bsink,
   ClutterGstVideoSink *sink;
   sink = CLUTTER_GST_VIDEO_SINK (bsink);
 
+  GST_DEBUG_OBJECT (bsink, "Getting caps for %s", gst_caps_to_string (filter));
+
   if (sink->priv->caps == NULL)
     return NULL;
-  else
-    return gst_caps_ref (sink->priv->caps);
+
+  /* if (filter != NULL) */
+  /*   return gst_caps_intersect (sink->priv->caps, filter); */
+  return gst_caps_ref (sink->priv->caps);
 }
 
 static gboolean
@@ -1831,6 +1872,9 @@ clutter_gst_video_sink_parse_caps (GstCaps *caps,
 
   if (save)
     {
+
+      GST_INFO_OBJECT (sink, "saving infos");
+
       priv->info = vinfo;
 
       priv->format = format;
@@ -1878,6 +1922,8 @@ clutter_gst_video_sink_set_caps (GstBaseSink *bsink,
   sink = CLUTTER_GST_VIDEO_SINK (bsink);
   priv = sink->priv;
 
+  GST_INFO_OBJECT (bsink, "Setting caps to %s", gst_caps_to_string (caps));
+
   if (!clutter_gst_video_sink_parse_caps (caps, sink, FALSE))
     return FALSE;
 
@@ -1911,6 +1957,7 @@ clutter_gst_source_dispatch (GSource *source,
 
       gst_source->has_new_caps = FALSE;
 
+      clear_frame_textures (gst_source->sink);
       dirty_default_pipeline (gst_source->sink);
 
       /* We are now in a state where we could generate the pipeline if
@@ -1930,10 +1977,14 @@ clutter_gst_source_dispatch (GSource *source,
       clutter_gst_video_sink_upload_overlay (gst_source->sink, buffer);
 
       if (gst_buffer_get_video_gl_texture_upload_meta (buffer) != NULL) {
+        GST_DEBUG_OBJECT (gst_source->sink,
+                          "Trying to upload buffer %p with GL", buffer);
         if (!priv->renderer->upload_gl (gst_source->sink, buffer)) {
           goto fail_upload;
         }
       } else {
+        GST_DEBUG_OBJECT (gst_source->sink,
+                          "Trying to upload buffer %p with software", buffer);
         if (!priv->renderer->upload (gst_source->sink, buffer))
           goto fail_upload;
       }
@@ -2133,12 +2184,33 @@ clutter_gst_video_sink_start (GstBaseSink *base_sink)
   ClutterGstVideoSink *sink = CLUTTER_GST_VIDEO_SINK (base_sink);
   ClutterGstVideoSinkPrivate *priv = sink->priv;
 
+  GST_INFO_OBJECT (sink, "Start");
+
   priv->source = clutter_gst_source_new (sink);
   g_source_attach ((GSource *) priv->source, NULL);
   priv->flow_return = GST_FLOW_OK;
   return TRUE;
 }
 
+static gboolean
+clutter_gst_video_sink_stop (GstBaseSink *base_sink)
+{
+  ClutterGstVideoSink *sink = CLUTTER_GST_VIDEO_SINK (base_sink);
+  ClutterGstVideoSinkPrivate *priv = sink->priv;
+
+  GST_INFO_OBJECT (sink, "Stop");
+
+  if (priv->source)
+    {
+      GSource *source = (GSource *) priv->source;
+      g_source_destroy (source);
+      g_source_unref (source);
+      priv->source = NULL;
+    }
+
+  return TRUE;
+}
+
 static void
 clutter_gst_video_sink_set_property (GObject *object,
                                      unsigned int prop_id,
@@ -2178,23 +2250,6 @@ clutter_gst_video_sink_get_property (GObject *object,
     }
 }
 
-static gboolean
-clutter_gst_video_sink_stop (GstBaseSink *base_sink)
-{
-  ClutterGstVideoSink *sink = CLUTTER_GST_VIDEO_SINK (base_sink);
-  ClutterGstVideoSinkPrivate *priv = sink->priv;
-
-  if (priv->source)
-    {
-      GSource *source = (GSource *) priv->source;
-      g_source_destroy (source);
-      g_source_unref (source);
-      priv->source = NULL;
-    }
-
-  return TRUE;
-}
-
 static gboolean
 clutter_gst_video_sink_propose_allocation (GstBaseSink *base_sink, GstQuery *query)
 {
@@ -2205,8 +2260,11 @@ clutter_gst_video_sink_propose_allocation (GstBaseSink *base_sink, GstQuery *que
 
   gst_query_add_allocation_meta (query,
                                  GST_VIDEO_META_API_TYPE, NULL);
+#ifdef HAVE_GL_TEXTURE_UPLOAD
+
   gst_query_add_allocation_meta (query,
                                  GST_VIDEO_GL_TEXTURE_UPLOAD_META_API_TYPE, NULL);
+#endif
   gst_query_add_allocation_meta (query,
                                  GST_VIDEO_OVERLAY_COMPOSITION_META_API_TYPE, NULL);
 
@@ -2223,6 +2281,11 @@ clutter_gst_video_sink_class_init (ClutterGstVideoSinkClass *klass)
   GstPadTemplate *pad_template;
   GParamSpec *pspec;
 
+  GST_DEBUG_CATEGORY_INIT (clutter_gst_video_sink_debug,
+                           "cluttervideosink",
+                           0,
+                           "clutter video sink");
+
   g_type_class_add_private (klass, sizeof (ClutterGstVideoSinkPrivate));
   go_class->set_property = clutter_gst_video_sink_set_property;
   go_class->get_property = clutter_gst_video_sink_get_property;
